unit Util;

{******************************************************************************
 Nome:			Util
 Scopo:			Costanti, tipi e procedure di utilit generale.
 Generalit:Util contiene costanti, tipi e procedure di utilit generale di
						complessit non sufficiente a giustificare una unit dedicata.
 Autore:    Ing. M. Venturini
 Revisore:
 Data:      11 Mar. 96
 Revisioni: 22 Dec. 1997: porting a 32 bit.
******************************************************************************}

interface

uses
	SysUtils, Forms;

const
	MaxShortInt = 127;
  MaxSmallInt = 32767;
  MaxLongint = MaxInt;
  MaxByte = 255;
  MaxWord = 65535;
  MaxInteger = MaxInt;
  MaxCardinal = MaxInt;
  BitInShortInt = 8;
  BitInSmallInt = 16;
  BitInLongInt = 32;
	BitInByte = 8;
  BitInWord = 16;
  BitInInteger = 32;
  BitInCardinal = 31;
  mSecinSec = 1000;
  EInternalFmt = '%s - %d'; {Formato per la presentazione della exception EInternal}

type
  TWindowsInfoFlagsEnum = (wif3x,wif95,wif98,wifNT,wif16,wif32,wifStyle3x,wifStyleNew);
  TWindowsInfoFlags = set of TWindowsInfoFlagsEnum;
	EInternal = class(Exception);

function RunningAsInherited(const	Instance: TObject;
														const StaticClass: TClass): Boolean;
function ParseStr(var  	ParseBuffer: string;
									const	TargetKey: Char): string;
function ParseStrAndTrim(var  	ParseBuffer: string;
												 const	TargetKey: Char): string;
function TrimmedParseStr(var 	 ParseBuffer: string;
												 const TargetKey: Char): string;
function StrOfChar(const C: Char;
									 const N: Integer): string;
function BlankStr(const N: Integer): string;
procedure Delay(const Interval: Longint);
procedure HideTitleBar(Form: TForm);
procedure ForceMinimizeBorderIcon(Form: TForm);
function WindowsInfoFlags: TWindowsInfoFlags;

{*****************************************************************************}

implementation

uses
	Windows;

function RunningAsInherited(const	Instance: TObject;
														const StaticClass: TClass): Boolean;
begin

	Result := (Instance is StaticClass) and not (Instance.ClassType = StaticClass);

end;

{******************************************************************************
 --	function ParseStr
 Esegue il parsing di una stringa.
 --	Parametri:
		ParseBuffer: 		in out string
										Buffer con la striga su cui eseguire il parse all'ingresso
										e stringa residua all'uscita.
		TargetKey:			in Char
										Carattere di separazione dei token.
		Result:					out string
										Token risultato del parsing.
 ParseBuffer  assunto avere il formato [Head][[TargetKey][Tail]]. Head viene
 ritornato in Result e Tail in ParseBuffer. Se TargetKey  assente per Tail si
 assume ''.
******************************************************************************}

function ParseStr(var 	ParseBuffer: string;
									const TargetKey: Char): string;

var
	I: Integer;

begin

	I := Pos(TargetKey, ParseBuffer);
	If I = 0 Then Begin
		Result := ParseBuffer;
		ParseBuffer := '';
	end
	else begin
		Result := Copy(ParseBuffer,1,I - 1);
		Delete(ParseBuffer,1,I);
	end;

end;

{******************************************************************************
 --	function ParseStrAndTrim
 Esegue il parsing di una stringa e quindi esegue la Trim sul residuo.
 --	Parametri:
		ParseBuffer: 		in out string
										Buffer con la striga su cui eseguire il parse all'ingresso
										e stringa residua all'uscita.
		TargetKey:			in Char
										Carattere di separazione dei token.
		Result:					out string
										Token risultato del parsing.
 ParseBuffer  assunto avere il formato [Head][[TargetKey][Tail]]. Head viene
 ritornato in Result e Tail in ParseBuffer. Se TargetKey  assente per Tail si
 assume ''. Su Tail viene esguita la Trim.
******************************************************************************}

function ParseStrAndTrim(var 	 ParseBuffer: string;
												 const TargetKey: Char): string;
begin

	Result := ParseStr(ParseBuffer,TargetKey);
  ParseBuffer := Trim(ParseBuffer);

end;

{******************************************************************************
 --	function TrimmedParseStr
 Esegue il parsing di una stringa ed esegue la Trim sul risultato.
 --	Parametri:
		ParseBuffer: 		in out string
										Buffer con la striga su cui eseguire il parse all'ingresso
										e stringa residua all'uscita.
		TargetKey:			in Char
										Carattere di separazione dei token.
		Result:					out string
										Token risultato del parsing.
 ParseBuffer  assunto avere il formato [Head][[TargetKey][Tail]]. Head viene
 ritornato in Result e Tail in ParseBuffer. Se TargetKey  assente per Tail si
 assume ''. Su Result viene esguita la Trim.
******************************************************************************}

function TrimmedParseStr(var 	 ParseBuffer: string;
												 const TargetKey: Char): string;
begin

	Result := Trim(ParseStr(ParseBuffer,TargetKey));

end;

{******************************************************************************
 --	function StrOfChar
 Ritorna una stringa composta di N caratteri C.
******************************************************************************}

function StrOfChar(const C: Char;
									 const N: Integer): string;
begin

	if N > 0 then begin
  	SetLength(Result,N);
	  FillChar(Result[1],N,Ord(C));
  end
  else
  	Result := '';

end;

{******************************************************************************
 --	function StrOfChar
 Ritorna una stringa composta di N blank.
******************************************************************************}

function BlankStr(const N: Integer): string;
begin

	Result := StrOfChar(' ',N);

end;

{******************************************************************************
 --	Procedure Delay
 Restituisce il controllo al chiamante dopo il tempo specificato.
 --	Parametri:
		Interval:				in Longint
                    Ritardo richiesto in millisecondi.
******************************************************************************}

procedure Delay(const Interval: Longint);

var
  FirstTickCount: Longint;

begin

  FirstTickCount:= GetTickCount;
  repeat
    Application.ProcessMessages;
  until ((GetTickCount - FirstTickCount) >= Interval);

end;

{******************************************************************************
 -- Procedure HideTitleBar
 Rimuove da un form la TitleBar che comprende il System Menu, il Minimize Cmd
 e la Caption.
 --	Parametri:
		Form:     			in TForm
										Il form su cui agire.
******************************************************************************}

procedure HideTitleBar(Form: TForm);

var
	Save: Longint;

begin

	Save := GetWindowLong(Form.Handle,gwl_Style);
  if (Save and ws_Caption) = ws_Caption then begin
  	SetWindowLong(Form.Handle,gwl_Style,Save and not ws_Caption or ws_Border);
	  Form.Height := Form.Height - GetSystemMetrics(sm_cyCaption);
  	Form.Refresh;
  end;

end;

{******************************************************************************
 -- Procedure ForceMinimizeBorderIcon
 Forza la presenza del Minimize Cmd anche sui Form che non lo prevedono
 (dialoghi).
 --	Parametri:
		Form:     			in TForm
										Il form su cui agire.
******************************************************************************}

procedure ForceMinimizeBorderIcon(Form: TForm);

var
	Save: Longint;

begin

	Save := GetWindowLong(Form.Handle,gwl_Style);
  SetWindowLong(Form.Handle,gwl_Style,Save or ws_MinimizeBox);

end;

{******************************************************************************
 -- Function WindowsInfoFlags
 Ritorna informazioni relative al tipo ed alla versione di Windows.
 --	Parametri:
		Result:    			out TWindowsInfoFlags
******************************************************************************}

function WindowsInfoFlags: TWindowsInfoFlags;

const
	BufferSize = 31;
	TargetName = 'OS';
  TargetValue = 'Windows_NT';

var
  EnvP: PChar;

begin

	Result := [wif3x,wif16,wifStyle3x];
  EnvP := StrAlloc(BufferSize + 1);
  GetEnvironmentVariable(PChar(TargetName),EnvP,BufferSize);
 	if StrIComp(EnvP,PChar(TargetValue)) = 0 then
		Result := [wifNT,wif16,wif32,wifStyleNew];
  StrDispose(EnvP);

end;


end.

